#!/usr/bin/env python
'''
Independent validation by CCLE data: only 14 drugs overlapped, for each overlapped drug, output a joint plot.
'''
import math
import numpy as np
import pandas as pd
import seaborn as sns
from scipy import stats
from pandas import DataFrame as df
from matplotlib import pyplot as plt
import matplotlib
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42

__Author__ = 'Yue Wang'
__Date__ = 'Jan 31, 2017'


'''
Read response of overlapped drugs
'''
gdsc_res = pd.read_csv(
    'drug_res/drug_res_merge.csv',
    index_col=[0, 1], header=0, sep='\t').dropna(axis=0, how='any')
drug_list = list(pd.read_csv('overlapped/overlap_cell_lines_between_two.csv', header=0, index_col=0, sep='\t').index)
drug_id = pd.read_csv('overlapped/drug_list.csv', header=0, index_col=0, sep=',')
performance = df(index=drug_list, columns=['Correlation between predicted and observed CCLE IC50', '-Log10(pval)', 'Correlation between GDSC and CCLE IC50'])
for drug in drug_list:
    drug_res = df(pd.read_csv('overlapped/LN_IC50/' + drug + '_' + str(drug_id.loc[drug, 'COSMIC']) + '.csv', header=0, index_col=0, sep=',').loc[:, 'CCLE LN_IC50'])
    drug_res.index = drug_res.index.astype(str)
    current_gdsc = df(gdsc_res.loc[drug_id.loc[drug, 'COSMIC']]['LN_IC50'])
    current_gdsc.index = current_gdsc.index.astype(str)

    # get predicted value
    predicted = pd.read_csv('Elastic_net_para/predicted_values/LN_IC50_' + str(drug_id.loc[drug, 'COSMIC']) + '.csv', header=0, sep='\t')
    # get predicted mean
    predicted_mean = predicted.copy()
    predicted_mean = df(predicted_mean.mean(), columns=['predicted'])

    # get intersection of cell lines
    cell_line = set(list(drug_res.index)).intersection(list(predicted_mean.index))

    oe_frame = df()
    oe_frame['Observed CCLE Drug Response (IC50)'] = drug_res.loc[cell_line, 'CCLE LN_IC50']
    oe_frame['Predicted Drug Response (IC50)'] = predicted_mean.loc[cell_line, 'predicted']
    oe_frame['Observed GDSC Drug Response (IC50)'] = current_gdsc.loc[cell_line, 'LN_IC50']
    # print oe_frame.describe()
    # oe_frame = df(oe_frame[oe_frame < 2.079442]).dropna(axis=0, how='any')
    # print oe_frame
    if len(oe_frame.index) <= 5:
        continue
    else:
        stat, p = stats.spearmanr(oe_frame['Observed CCLE Drug Response (IC50)'], oe_frame['Predicted Drug Response (IC50)'])
        performance.at[drug, 'Correlation between predicted and observed CCLE IC50'] = stat
        performance.at[drug, '-Log10(pval)'] = -math.log(p, 10)

        stat, p = stats.spearmanr(oe_frame['Observed CCLE Drug Response (IC50)'], oe_frame['Observed GDSC Drug Response (IC50)'])
        performance.at[drug, 'Correlation between GDSC and CCLE IC50'] = stat
    # fig = plt.figure()
    # sns.set_style('white')
    # sns.jointplot(x='Observed CCLE Drug Response (IC50)', y='Predicted Drug Response (IC50)', data=oe_frame, kind='kde', stat_func=stats.spearmanr, color="r")

    # fig = plt.gcf()
    # plt.subplots_adjust(top=.9)
    # fig.suptitle('Model Indepent Validation of Drug ' + drug + ' (IC50 vs IC50)', fontsize=14)
    # fig.savefig('independent_validation/CCLE/LN_IC50/' + drug + '_jointplot.pdf')
    # fig.savefig('Panels/Fig3_Model/CCLE/' + drug + '.pdf', transparent=True)
drug_name = pd.read_csv(
    'drug_res/drug_id2name.csv', sep='\t',
    header=0, index_col=0)
performance = performance.rename(index=drug_name['DRUG NAME'])
performance.to_csv('NCOMM_revision/independent_validation/CCLE_performance.csv', sep=',')
sns.jointplot(
    x='Correlation between predicted and observed CCLE IC50', y='Correlation between GDSC and CCLE IC50', data=performance)
fig = plt.gcf()
fig.savefig('NCOMM_revision/independent_validation/CCLE_GDSC.pdf', transparent='True')
assert 0 == 1
fig = plt.figure()
sns.lmplot(
    x='Spearman coef.', y='-Log10(pval)', data=performance,
    fit_reg=False, scatter=True, scatter_kws={'color': 'salmon'})

fig = plt.gcf()
plt.show()
# fig.savefig('Panels/FigS3/D_ccle_overall.pdf', transparent='True')
# performance.to_csv('Panels/FigS3/D_annotation.csv', sep=',')
