#!/usr/bin/env python
'''
Independent validation by CTRP data: only 14 drugs overlapped, for each overlapped drug, output a joint plot.
'''
import math
import numpy as np
import pandas as pd
import seaborn as sns
from scipy import stats
from pandas import DataFrame as df
from matplotlib import pyplot as plt
import matplotlib
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42

__Author__ = 'Yue Wang'
__Date__ = 'April 31, 2018'


'''
Read response of overlapped drugs
'''

drug_res = pd.read_csv(
    'NCOMM_revision/specificity/ctrp_op_agent_drug_res.csv',
    index_col=[0, 1], header=0, sep=',').dropna(axis=0, how='any')

gdsc_res = pd.read_csv(
    'drug_res/drug_res_merge.csv',
    index_col=[0, 1], header=0, sep='\t').dropna(axis=0, how='any')

drug_list = [d for (d, c) in drug_res.index]
drug_list = list(set(drug_list))

performance = df(index=drug_list, columns=['Correlation between predicted and observed CTRP AUC', '-Log10(pval)', 'Correlation between GDSC and CTRP AUC'])
for drug in drug_list:
    current_res = df(drug_res.loc[drug]['AUC_CTRP'])
    current_res.index = current_res.index.astype(str)
    current_gdsc = df(gdsc_res.loc[drug]['AUC'])
    current_gdsc.index = current_gdsc.index.astype(str)

    # get predicted value
    predicted = pd.read_csv('Elastic_net_para/predicted_values/AUC_' + str(drug) + '.csv', header=0, sep='\t')
    # get predicted mean
    predicted_mean = predicted.copy()
    predicted_mean = df(predicted_mean.mean(), columns=['predicted'])

    # get intersection of cell lines
    cell_line = set(list(current_res.index)).intersection(list(predicted_mean.index))

    oe_frame = df()
    oe_frame['Observed CTRP Drug Response (AUC)'] = current_res.loc[cell_line, 'AUC_CTRP']
    oe_frame['Predicted Drug Response (AUC)'] = predicted_mean.loc[cell_line, 'predicted']
    oe_frame['Observed GDSC Drug Response (AUC)'] = current_gdsc.loc[cell_line, 'AUC']
    # print oe_frame.describe()
    # oe_frame = df(oe_frame[oe_frame < 2.079442]).dropna(axis=0, how='any')
    # print oe_frame
    if len(oe_frame.index) <= 5:
        continue
    else:
        stat, p = stats.spearmanr(oe_frame['Observed CTRP Drug Response (AUC)'], oe_frame['Predicted Drug Response (AUC)'])
        performance.at[drug, 'Correlation between predicted and observed CTRP AUC'] = stat
        performance.at[drug, '-Log10(pval)'] = -math.log(p, 10)

        stat, p = stats.spearmanr(oe_frame['Observed CTRP Drug Response (AUC)'], oe_frame['Observed GDSC Drug Response (AUC)'])
        performance.at[drug, 'Correlation between GDSC and CTRP AUC'] = stat
    # fig = plt.figure()
    '''
    sns.set_style('white')
    sns.jointplot(x='Observed CTRP Drug Response (AUC)', y='Predicted Drug Response (AUC)', data=oe_frame, kind='reg', stat_func=stats.spearmanr, color="r")

    fig = plt.gcf()
    plt.subplots_adjust(top=.9)
    fig.suptitle('Model Indepent Validation of Drug ' + str(drug) + ' (AUC vs AUC)', fontsize=14)
    fig.savefig('independent_validation/CTRP/AUC/' + str(drug) + '_jointplot.png')
    plt.close()
    # fig.savefig('Panels/Fig3_Model/CTRP/' + drug + '.pdf', transparent=True)
    '''
# print performance.sort_values(by='-Log10(pval)', ascending=True)
# print performance[performance['-Log10(pval)'] >= -math.log(0.05, 10)]
drug_name = pd.read_csv(
    'drug_res/drug_id2name.csv', sep='\t',
    header=0, index_col=0)
performance = performance.rename(index=drug_name['DRUG NAME'])
performance.to_csv('NCOMM_revision/independent_validation/CTRP_performance.csv', sep=',')
sns.jointplot(
    x='Correlation between predicted and observed CTRP AUC', y='Correlation between GDSC and CTRP AUC', data=performance)
fig = plt.gcf()
fig.savefig('NCOMM_revision/independent_validation/CTRP_GDSC.pdf', transparent='True')
assert 0 == 1
sns.lmplot(
    x='Correlation between predicted and observed CTRP AUC', y='-Log10(pval)', data=performance,
    fit_reg=False, scatter=True, scatter_kws={'color': 'salmon'})
plt.axhline(y=-math.log(0.05, 10), ls=':', c='k')
plt.axvline(x=0.0, ls=':', c='k')
fig = plt.gcf()
# plt.show()
# fig.savefig('Panels/FigS3/D_CTRP_overall.pdf', transparent='True')
fig.savefig('independent_validation/CTRP/CTRP_AUC_all.png')
# performance.to_csv('Panels/FigS3/D_annotation.csv', sep=',')
