#!/usr/bin/env python
'''
Prepare the model validation in TCGA.
Input:
    lncRNA expression matrix in patient samples by cancer types
    Top predictors along with their coefficients [pan and cancer-specific]
Output:
    Predicted IC50 matrix for pan and cancer-specific: row = drug id, col = patient sample
'''

import pandas as pd

from pandas import DataFrame as df
from os import listdir as ls

__Author__ = 'Yue Wang'
__Date__ = 'Feb 27, 2017'


'''
Read in patient expression data.
'''
patientDir = ls('independent_validation/TCGA/patient_expression/')

# panCan model, need to merge all sample in order to apply z-score transformation
patientExpr = pd.read_csv('independent_validation/TCGA/patient_expression/' + patientDir[0], header=0, index_col=0, sep=',')
for d in patientDir[1:]:
    curExpr = pd.read_csv('independent_validation/TCGA/patient_expression/' + d, header=0, index_col=0, sep=',')
    patientExpr[curExpr.columns] = curExpr.loc[:, curExpr.columns]
patientExpr = patientExpr.fillna(0)

# z-score conversion
patientExpr['mean'] = patientExpr.mean(1)

# get std
patientExpr['std'] = patientExpr.std(1)

# z-score: (x - mean)/std
expr_z = patientExpr.copy()

for cell in expr_z.columns[:-2]:
    expr_z.loc[:, cell] = (expr_z.loc[:, cell] - expr_z.loc[:, 'mean']) / expr_z.loc[:, 'std']

patientExpr = df(expr_z.loc[:, patientExpr.columns[:-2]])

'''
Read in predictors and the corresponding coefficients, output the predicted ic50
'''
drugCoefDir = ls('Elastic_net_para/final_model_coefficients/LN_IC50/')
drugList = [x.split('.')[0] for x in drugCoefDir]
# initialize the output frame
predIC50 = df(index=drugList, columns=patientExpr.columns)
for drug in drugCoefDir:
    coefExpr = pd.read_csv('Elastic_net_para/final_model_coefficients/LN_IC50/' + drug, header=0, index_col=0, sep='\t')
    for patient in list(patientExpr.columns):
        coefExpr['Expression'] = patientExpr.loc[coefExpr.index, patient]
        coefExpr['Prediction'] = coefExpr.Expression * coefExpr.average_coefficient
        predIC50.at[drug.split('.')[0], patient] = coefExpr['Prediction'].sum()

predIC50.to_csv('independent_validation/TCGA/pred_ic50_patient.csv', sep=',')
